/*
 * Process execution bits for PKUnity SoC and UniCore ISA
 *
 * Copyright (C) 2001-2012 GUAN Xue-tao
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */
#ifndef __UNICORE_EXEC_H__
#define __UNICORE_EXEC_H__

#define arch_align_stack(x)		(x)

#endif /* __UNICORE_EXEC_H__ */
