/*
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License version 2 as published
 *  by the Free Software Foundation.
 *
 *  Copyright (C) 2010 John Crispin <blogic@openwrt.org>
 */

#ifndef __LANTIQ_IRQ_H
#define __LANTIQ_IRQ_H

#include <lantiq_irq.h>

#define NR_IRQS 256

#include_next <irq.h>

#endif
